@@ -3,6 +3,9 @@
 #ifndef liqcell_H
 #define liqcell_H
 
+#include <unistd.h>
+#include <stdarg.h>
+
 #include "liqbase.h"
 //#include "liqcell_easyrun.h"
 //#include "liqcell_prop.h"
@@ -132,7 +135,7 @@ liqcell*  liqcell_child_lookup(liqcell *self,char *name); // use dotted branches
 liqcell*  liqcell_child_lookup_simple(liqcell *self,char *name);	// ignore dotted branches
 liqcell*  liqcell_child_lookup_nameclass(liqcell *self,char *name,char *classname);
 
-
+liqcell *liqcell_lastchild(liqcell *self);
 
 liqcell *	liqcell_getlinkparent(liqcell *self);
 liqcell *	liqcell_getlinkprev(liqcell *self);
@@ -184,6 +187,12 @@ char *	liqcell_getcaption(liqcell *self);
 
 
 
+void 	liqcell_setcaption_vprintf(liqcell *self,char *format, va_list arg);
+void 	liqcell_setcaption_printf(liqcell *self,char *format, ...);
+
+
+
+
 void 	liqcell_setclassname(liqcell *self,char *classname);	// class name used to construct
 char *	liqcell_getclassname(liqcell *self);
 
@@ -268,6 +277,9 @@ int    	liqcell_getx(liqcell *self);
 int    	liqcell_gety(liqcell *self);
 int    	liqcell_getw(liqcell *self);
 int    	liqcell_geth(liqcell *self);
+int    	liqcell_getcx(liqcell *self);
+int    	liqcell_getcy(liqcell *self);
+
 int    	liqcell_getinnerw(liqcell *self);
 int    	liqcell_getinnerh(liqcell *self);
 int    	liqcell_getkineticx(liqcell *self);
@@ -281,10 +293,14 @@ int     liqcell_ensurevisible_centred(liqcell *self);
 void 	liqcell_zorder_totop(liqcell *self);  // moves the cell to the top of the zorder, NULL function at present
 liqcell * liqcell_getbasewidget(liqcell *self); // called from within an event steps backwards until it finds the base widget this item was created by
 
+void liqcell_print(liqcell *self,char *title,int recdep);
+void liqcell_print2(liqcell *self);
+
 //######################################################################### Arrangement and Layout tools
 
 
 int     liqcell_child_countvisible(liqcell *self);
+int     liqcell_child_countselected(liqcell *self);
 
 int 	liqcell_child_arrange_easytile(liqcell *self);			// make sure all contents are bound within the area
 int 	liqcell_child_arrange_easyrow(liqcell *self);			// split into Left|Centre|Right boxes (newspaper panels)
