@@ -35,8 +35,19 @@
 //blendpix.Red = (srcpixone.Red - srcpixtwo.Red) * alpha + srcpixtwo.Red) \ 100
 
 
+// 20090709_001938 lcuk : added to test the thickline and antialiased line capability
+// 20090709_001954 lcuk : must extend checking, i think its viable :)
+// 20090709_002008 lcuk : todo formulate parameter modifications and entry points
+// 20090709_002033 lcuk : decide whether to allow native current single line sources to use standard unalised sections
+// 20090709_002053 lcuk : allow sketches themselves to choose aa or not, or rather give the user the option to choose for them
+
+//#define USE_AA_THICKLINE 1
+//#define USE_AA 1
+
 
 #include <memory.h>
+#include <stdlib.h> 
+
 
 // http://www.x.org/docs/Xv/video
 // todo:lcuk: damn all this time i have been calculating planes and offsets and sizes i could have used properties from the liqimage itself.
@@ -1201,7 +1212,8 @@ void xsurface_drawzoomblendimage(
 
 	// jump if expected dimensions match image dimensions and we are not doing any blending
 
-	if(smw==diw && smh==dih && blend==255 && srcimage->num_planes==3)
+	if(smw==diw && smh==dih && blend==255 && srcimage->num_planes==3
+	   && six==0 && siy==0 && siw==diw && sih==dih)			// 20090812_235925 lcuk : extra round of checks to prevent exactly screensized images from being scaled
 	//if(siw==diw && sih==dih && blend==255 && srcimage->num_planes==3)
 	{
 
@@ -1501,7 +1513,7 @@ inline void xsurface_drawfadeoutrect_yuv(liqimage *surface,int x,int y,int w,int
 		h=(surface->height-y);
 	}
     
-unsigned char spreaddiv2 = spread/2;
+//unsigned char spreaddiv2 = spread/2;
     
 	
 
@@ -1509,7 +1521,7 @@ unsigned char spreaddiv2 = spread/2;
 //	grey4=grey<<24 | grey<<16 | grey<<8 | grey;
 	register unsigned int xx,yy;
 	register unsigned char *pdata;
-	register unsigned int *epdata;
+	//register unsigned int *epdata;
 	for (yy = y; yy < (y+h); yy++)
 	{
 		pdata = &surface->data[ surface->offsets[0] + (surface->width*yy) + x ];
@@ -1562,9 +1574,10 @@ unsigned char spreaddiv2 = spread/2;
 	u2=u<<8 | u;
 	v2=v<<8 | v;
 	register unsigned char *udata;
-	register unsigned short *eudata;
+	//register unsigned short *eudata;
 	register unsigned char *vdata;
-	register unsigned short *evdata;
+	//register unsigned short *evdata;
+	
 	unsigned int pw = surface->width;
 	unsigned int ph = surface->height;
 	unsigned int uo = surface->offsets[1];//pw*ph;
@@ -1921,7 +1934,7 @@ unsigned int pw=surface->width;
 
 #define interal_linepaintchar(x,y,c,buffer,linewidth)  buffer[ (linewidth) * (y) + (x) ] = (c)
 
-void xsurface_interalline_noaa(liqimage *surface,int x1, int y1, int x2, int y2, char grey,char *buffer,int linewidth)
+inline void xsurface_interalline_noaa(liqimage *surface,int x1, int y1, int x2, int y2, char grey,char *buffer,int linewidth)
 {
                 // 20090520_012704 lcuk : boundary is already confirmed as existing within the buffer
                 // 20090520_012727 lcuk : this is a greyscale line and it exhibits aliasing effects
@@ -1984,6 +1997,302 @@ void xsurface_interalline_noaa(liqimage *surface,int x1, int y1, int x2, int y2,
 
 
 
+ // dont like this though, its close but still feels more messy than direct pixels
+ // might do something about that
+ // also, ensure only the grey channel goes through here, theres no real need to smear the UV any further
+
+
+// from a vb class here:
+// http://www.bigresource.com/VB-Anti-aliasing-text-8vUifLfKBe.html
+
+/*
+#define kkointeral_linepaintchar_alpha(x,y,c,alpha,buffer,linewidth) \
+{ int off = (linewidth) * (y) + (x);  int b=buffer[ off ]; int cc=(c); float a=(alpha); buffer[ off ] = (b) + ((float)((cc)-(b)) * (a)); }
+
+
+
+#define interal_linepaintchar_alpha(x,y,c,alpha,buffer,linewidth) \
+{ int off = (linewidth) * (y) + (x);  int b=buffer[ off ]; int cc=(c); float a=(alpha); buffer[ off ] = (b*(1.0-a)) + (cc*(a)); }
+
+
+#define llinteral_linepaintchar_alpha(x,y,c,alpha,buffer,linewidth) \
+{ int off = (linewidth) * (y) + (x);  int b=buffer[ off ]; buffer[ off ] = (c) ; }
+ */
+
+#define interal_linepaintchar_alpha(x,y,c,alpha,buffer,linewidth) \
+{ int off = (linewidth) * (y) + (x);  int b=(unsigned char)buffer[ off ]; int cc=(unsigned char)(c); float a=(alpha); buffer[ off ] = b + (cc-b)*a; }
+
+inline float fracpart(float a)
+{
+	float b=((int)(a));
+      
+	return a - b;
+}
+
+
+
+
+
+
+inline void xsurface_interalline_aa(liqimage *surface,int x1, int y1, int x2, int y2, char grey,char *buffer,int linewidth)
+{
+float deltax, deltay;
+int loopc;
+int start , finish;
+float dx, dy, dydx;
+
+	//grey=255;
+
+
+//char LR , LG, LB;
+	//	xsurface_interalline_noaa(surface,x1,y1,x2,y2,grey/2,buffer,linewidth);
+
+	deltax = abs(x2 - x1); //' Calculate deltax and deltay for initialisation
+	deltay = abs(y2 - y1);
+	// 20090708_211634 lcuk : do all right now during test
+#ifndef USE_AA_THICKLINE
+	if ((deltax != 0) && (deltay != 0))  //' it is not a horizontal or a vertical line
+#endif
+	{
+		//LR = (AColor And &HFF&)
+		//LG = (AColor And &HFF00&) / &H100&
+		//LB = (AColor And &HFF0000) / &H10000
+		if (deltax > deltay) //' horizontal or vertical
+		{
+			if (y2 > y1) //' determine rise and run
+			{
+				dydx = -(deltay / deltax);
+			}
+			else
+			{
+				dydx = deltay / deltax;
+			}
+			if (x2 < x1)
+			{
+				start = x2; //' right to left
+				finish = x1;
+				dy = y2;
+			}
+			else
+			{
+				start = x1; //' left to right
+				finish = x2;
+				dy = y1;
+				dydx = -dydx; //' inverse slope
+			}
+			for(loopc = start; loopc<finish;loopc++)
+			{
+				//liqapp_log("dy=%3.5f  (1.0-f(dy))=%3.5f  f(dy)=%3.5f",dy,1.0-fracpart(dy),fracpart(dy));
+				//AlphaBlendPixel (hdc, loopc, CInt(dy - 0.5), LR, LG, LB, 1 - FracPart(dy));
+				//AlphaBlendPixel (hdc, loopc, CInt(dy - 0.5) + 1, LR, LG, LB, FracPart(dy));
+                
+#ifdef USE_AA_THICKLINE
+				interal_linepaintchar_alpha(loopc,((int)(dy)) - 2,grey,1.0 - fracpart(dy),buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy)) - 1,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy))    ,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy)) + 1,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy)) + 2,grey,      fracpart(dy),buffer,linewidth);
+#else
+				interal_linepaintchar_alpha(loopc,((int)(dy))    ,grey,1.0 - fracpart(dy),buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy)) + 1,grey,      fracpart(dy),buffer,linewidth);
+#endif
+
+
+				dy = dy + dydx; //' next point
+			}
+
+		}
+		else
+		{
+			if(x2 > x1)  //' determine rise and run
+			{
+				dydx = -(deltax / deltay);
+			}
+			else
+			{
+				dydx = deltax / deltay;
+			}
+			if(y2 < y1)
+			{
+				start = y2; //' right to left
+				finish = y1;
+				dx = x2;
+			}
+			else
+			{
+				start = y1; //' left to right
+				finish = y2;
+				dx = x1;
+				dydx = -dydx; //' inverse slope
+			}
+			for(loopc = start; loopc<finish;loopc++)
+			{
+				//AlphaBlendPixel hdc, CInt(dx - 0.5), loopc, LR, LG, LB, 1 - FracPart(dx)
+				//AlphaBlendPixel hdc, CInt(dx - 0.5) + 1, loopc, LR, LG, LB, FracPart(dx)
+
+
+#ifdef USE_AA_THICKLINE
+				interal_linepaintchar_alpha(((int)(dx)) - 2,loopc,grey,1.0 - fracpart(dx),buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx)) - 1,loopc,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx))    ,loopc,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx)) + 1,loopc,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx)) + 2,loopc,grey,      fracpart(dx),buffer,linewidth);
+#else
+				interal_linepaintchar_alpha(((int)(dx))    ,loopc,grey,1.0 - fracpart(dx),buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx)) + 1,loopc,grey,      fracpart(dx),buffer,linewidth);
+#endif
+
+
+				dx = dx + dydx; //' next point
+			}
+
+		}
+
+	}
+#ifndef USE_AA_THICKLINE
+	else
+	{
+		xsurface_interalline_noaa(surface,x1,y1,x2,y2,grey,buffer,linewidth);
+	}
+#endif
+}
+
+
+
+
+inline void xsurface_interalline_aa_uv(liqimage *surface,int x1, int y1, int x2, int y2, char grey,char *buffer,int linewidth)
+{
+float deltax, deltay;
+int loopc;
+int start , finish;
+float dx, dy, dydx;
+
+	//grey=255;
+
+
+//char LR , LG, LB;
+	//	xsurface_interalline_noaa(surface,x1,y1,x2,y2,grey/2,buffer,linewidth);
+
+	deltax = abs(x2 - x1); //' Calculate deltax and deltay for initialisation
+	deltay = abs(y2 - y1);
+	// 20090708_211634 lcuk : do all right now during test
+
+#ifndef USE_AA_THICKLINE
+	if ((deltax != 0) && (deltay != 0))  //' it is not a horizontal or a vertical line
+#endif
+	{
+		//LR = (AColor And &HFF&)
+		//LG = (AColor And &HFF00&) / &H100&
+		//LB = (AColor And &HFF0000) / &H10000
+		if (deltax > deltay) //' horizontal or vertical
+		{
+			if (y2 > y1) //' determine rise and run
+			{
+				dydx = -(deltay / deltax);
+			}
+			else
+			{
+				dydx = deltay / deltax;
+			}
+			if (x2 < x1)
+			{
+				start = x2; //' right to left
+				finish = x1;
+				dy = y2;
+			}
+			else
+			{
+				start = x1; //' left to right
+				finish = x2;
+				dy = y1;
+				dydx = -dydx; //' inverse slope
+			}
+			for(loopc = start; loopc<finish;loopc++)
+			{
+				//liqapp_log("dy=%3.5f  (1.0-f(dy))=%3.5f  f(dy)=%3.5f",dy,1.0-fracpart(dy),fracpart(dy));
+				//AlphaBlendPixel (hdc, loopc, CInt(dy - 0.5), LR, LG, LB, 1 - FracPart(dy));
+				//AlphaBlendPixel (hdc, loopc, CInt(dy - 0.5) + 1, LR, LG, LB, FracPart(dy));
+                
+#ifdef USE_AA_THICKLINE
+				interal_linepaintchar_alpha(loopc,((int)(dy)) - 1,grey,1.0 - fracpart(dy),buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy))    ,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy)) + 1,grey,      fracpart(dy),buffer,linewidth);
+#else
+				interal_linepaintchar_alpha(loopc,((int)(dy))    ,grey,1.0 - fracpart(dy),buffer,linewidth);
+				interal_linepaintchar_alpha(loopc,((int)(dy)) + 1,grey,      fracpart(dy),buffer,linewidth);
+#endif
+
+
+				dy = dy + dydx; //' next point
+			}
+
+		}
+		else
+		{
+			if(x2 > x1)  //' determine rise and run
+			{
+				dydx = -(deltax / deltay);
+			}
+			else
+			{
+				dydx = deltax / deltay;
+			}
+			if(y2 < y1)
+			{
+				start = y2; //' right to left
+				finish = y1;
+				dx = x2;
+			}
+			else
+			{
+				start = y1; //' left to right
+				finish = y2;
+				dx = x1;
+				dydx = -dydx; //' inverse slope
+			}
+			for(loopc = start; loopc<finish;loopc++)
+			{
+				//AlphaBlendPixel hdc, CInt(dx - 0.5), loopc, LR, LG, LB, 1 - FracPart(dx)
+				//AlphaBlendPixel hdc, CInt(dx - 0.5) + 1, loopc, LR, LG, LB, FracPart(dx)
+
+
+#ifdef USE_AA_THICKLINE
+				interal_linepaintchar_alpha(((int)(dx)) - 1,loopc,grey,1.0 - fracpart(dx),buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx))    ,loopc,grey,1.0               ,buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx)) + 1,loopc,grey,      fracpart(dx),buffer,linewidth);
+#else
+				interal_linepaintchar_alpha(((int)(dx))    ,loopc,grey,1.0 - fracpart(dx),buffer,linewidth);
+				interal_linepaintchar_alpha(((int)(dx)) + 1,loopc,grey,      fracpart(dx),buffer,linewidth);
+#endif
+
+
+				dx = dx + dydx; //' next point
+			}
+
+		}
+
+	}
+#ifndef USE_AA_THICKLINE
+	else
+	{
+		xsurface_interalline_noaa(surface,x1,y1,x2,y2,grey,buffer,linewidth);
+	}
+#endif
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -2026,7 +2335,7 @@ void xsurface_interalline_noaa(liqimage *surface,int x1, int y1, int x2, int y2,
 
 #define interal_linepaintchar_invert(x,y,buffer,linewidth)  { unsigned char c=buffer[ (linewidth) * (y) + (x) ];buffer[ (linewidth) * (y) + (x) ] = 255-c; }
 
-void xsurface_interalline_invert(liqimage *surface,int x1, int y1, int x2, int y2, char *buffer,int linewidth)
+inline void xsurface_interalline_invert(liqimage *surface,int x1, int y1, int x2, int y2, char *buffer,int linewidth)
 {
 	int dx=x2-x1;		// distance
 	int dy=y2-y1;
@@ -2077,8 +2386,14 @@ void xsurface_interalline_invert(liqimage *surface,int x1, int y1, int x2, int y
 //######################################################################## linecolor
 //########################################################################
 
+// 20090708_213512 lcuk : better more direct way of setting or unsetting to test
+#ifdef USE_AA
+#define xsurface_interalline xsurface_interalline_aa
+#else
+#define xsurface_interalline xsurface_interalline_noaa
+#endif
 
-void xsurface_drawline_yuv(liqimage *surface,int x1, int y1, int x2, int y2, char grey,char u,char v)
+inline void xsurface_drawline_yuv(liqimage *surface,int x1, int y1, int x2, int y2, char grey,char u,char v)
 {
 	if(x1<0)return;
 	if(y1<0)return;
@@ -2097,22 +2412,33 @@ unsigned int pw=surface->width;
 	unsigned int uo = surface->offsets[1];//pw*ph;
 	unsigned int vo = surface->offsets[2];//uo + (uo >> 2);
 // if changing to aa, just take the grey for now, the uv variation fails
-	//xsurface_interalline(surface,x1,y1,   x2,y2,grey, &surface->data[surface->offsets[0]], pw);
-	xsurface_interalline_noaa(surface,x1,y1,   x2,y2,grey, (char *)&surface->data[surface->offsets[0]], pw);
+    // 20090708_203046 lcuk : try this once again
+	xsurface_interalline(surface,x1,y1,   x2,y2,grey, (char *)&surface->data[surface->offsets[0]], pw);
+	//xsurface_interalline_noaa(surface,x1,y1,   x2,y2,grey, (char *)&surface->data[surface->offsets[0]], pw);
 	x1>>=1;
 	y1>>=1;
 	x2>>=1;
 	y2>>=1;
 	pw>>=1;
-	xsurface_interalline_noaa(surface,x1,y1,   x2,y2,u   ,(char *)&surface->data[uo], pw);
-	xsurface_interalline_noaa(surface,x1,y1,   x2,y2,v   ,(char *)&surface->data[vo], pw);
+	//xsurface_interalline_noaa(surface,x1,y1,   x2,y2,u   ,(char *)&surface->data[uo], pw);
+	//xsurface_interalline_noaa(surface,x1,y1,   x2,y2,v   ,(char *)&surface->data[vo], pw);
+
+#ifdef USE_AA
+	xsurface_interalline_aa_uv(surface,x1,y1,   x2,y2,u   ,(char *)&surface->data[uo], pw);
+	xsurface_interalline_aa_uv(surface,x1,y1,   x2,y2,v   ,(char *)&surface->data[vo], pw);
+
+#else
+	xsurface_interalline(surface,x1,y1,   x2,y2,u   ,(char *)&surface->data[uo], pw);
+	xsurface_interalline(surface,x1,y1,   x2,y2,v   ,(char *)&surface->data[vo], pw);
+
+#endif
 }
 
 //########################################################################
 //######################################################################## linegrey
 //########################################################################
 
-void xsurface_drawline_grey(liqimage *surface,int x1, int y1, int x2, int y2, char grey)
+inline void xsurface_drawline_grey(liqimage *surface,int x1, int y1, int x2, int y2, char grey)
 {
 	if(x1<0)return;
 	if(y1<0)return;
@@ -2138,7 +2464,7 @@ unsigned int pw=surface->width;
 //######################################################################## linegreyinvert
 //########################################################################
 
-void xsurface_drawline_greyinv(liqimage *surface,int x1, int y1, int x2, int y2)
+inline void xsurface_drawline_greyinv(liqimage *surface,int x1, int y1, int x2, int y2)
 {
 	if(x1<0)return;
 	if(y1<0)return;
@@ -2181,7 +2507,7 @@ inline void interal_linepaintcharf(int x, int y,char c,char *buffer,int linewidt
 			interal_linepaintcharf(cx-y,cy-x,                         c,buffer,linewidth,numlines); \
 		}
 
-void xsurface_interalcircle(int cx, int cy, int r,char grey,char *buffer,int linewidth,int numlines)
+inline void xsurface_interalcircle(int cx, int cy, int r,char grey,char *buffer,int linewidth,int numlines)
 {
 int d=3-(2*r);
 int x=0;
@@ -2203,7 +2529,7 @@ int y=r;
 //######################################################################## circlegrey
 //########################################################################
 
-void xsurface_drawcircle_grey(liqimage *surface,int cx, int cy, int r,unsigned char grey)
+inline void xsurface_drawcircle_grey(liqimage *surface,int cx, int cy, int r,unsigned char grey)
 {
 unsigned int pw=surface->width;
 unsigned int ph=surface->height;
